<?php

/**
 * AppController class file serving as a base class for all controllers.
 *
 *
 * @author David M. Willink <developer99@ebox99.com>
 * @link http://www.developer99.com/
 * @copyright Copyright &copy; 2015 David M. Willink
 */
use Monolog\Logger;
use Monolog\Handler\RotatingFileHandler;
use Monolog\Handler\LogEntriesHandler;
// use Monolog\Handler\PushoverHandler;

class AppController
{
    /**
     * Tasks required before ANY route is followed.
     * @param  object   $f3 Framework variable
     * @return void
     */
    function beforeRoute($f3)
    {
        // Make sure when accessing this app that we are in secured mode.
        // TODO

        // Page timer .js is added to every page for now.
        $r = file_get_contents($f3->get('GLOBAL_ASSETS') . 'vendor/local/js/pagetimer.js');
        $r = str_ireplace('||||SessionExpire||||', $f3->get('PAGE_TIMEOUT') ,$r);
        $r = str_ireplace('||||LogoutURL||||', $f3->get('LEGACY_LOGOUT_URL') ,$r);
        $f3->set('page_timer_js', $r);

        // SET COOKIES FOR REQUEST COMING FROM LEGACY APP.
        // AND REDIRECT TO REQUESTED PAGE.
        // $f3->route('GET /legacyredir/@pagename/@appcookie', 'ReportController->viewReportsPage');
        // All /legacyredir/ routes will include user cookie info
        // to digest before launching page.
        if ( !(strpos($f3->get('URI'), 'legacyredir_Wf7Gb3B3xtyne') === FALSE) )
        {
            // See if we are coming from legacy IIS online app.
            // Set user cookies if so.
            if ( !empty($f3->get('PARAMS.appcookie')) )
            {
                $keys = explode('~', $f3->get('PARAMS.appcookie'));
                // ChainID
                $f3->set('COOKIE.userchain', substr($keys[0], 0, -1));
                // Chain Code ( ie. hmh )
                $f3->set('COOKIE.chaincode', substr($keys[1], 0, -1));
                // Session code string ( userid, sessionid, userpass, userlogin, currenttime, PIN (optional) )
                $f3->set('COOKIE.userid', substr($keys[2], 0, -1));
                // User name - Login name
                $f3->set('COOKIE.username', substr($keys[3], 0, -1));
                // User full name (first, last)
                $f3->set('COOKIE.userpwd', substr($keys[4], 0, -1));
                // User PIN for easy login
                $f3->set('COOKIE.userpin', substr($keys[5], 0, -1));
                // User full name (first, last)
                $f3->set('COOKIE.userfullname', substr($keys[6], 0, -1));

            }

            // Redirect to the page name requested.
            if ( !empty($f3->get('PARAMS.pagename')) )
            {
                // The pagename is actually the route name.  
                // So use the routename to reroute.
                // Since THIS request is to the legacy access point,
                // reset the ALIAS var.  The app uses it to know the page.
                $f3->set('ALIAS', $f3->get('PARAMS.pagename'));
                $f3->reroute('@' . $f3->get('PARAMS.pagename'));
                return;
            }

            // Else bad request.
            $f3->error(400);

        }

        // So far, all pages are from legacy system and will be for awhile.
        // If no cookies from legacy system, then 404.
        if ( empty($f3->get('COOKIE.chaincode')) )
        {
            $f3->error(404);
            return;
        }

        // Set the global system mode message.
        $f3->set('sys_mode_code', $f3->get('sys_mode_code_prod'));
        $f3->set('sys_mode_desc', $f3->get('sys_mode_desc_prod'));
        if ( ! (stripos($f3->get('URI'), '/demo/') === FALSE) )
        {
            $f3->set('sys_mode_code', $f3->get('sys_mode_code_demo'));
            $f3->set('sys_mode_desc', $f3->get('sys_mode_desc_demo'));
        }



        // echo $f3->get('URI');
        // exit;

        //
        // Turn on NEW labels
        // 
        $td = new DateTime();

        // NEW Airline Reconciliation Report
        $sd = new DateTime("2015-10-18");
        if ( $sd->diff($td)->days < 120 )
            $f3->set('new_report_label','&nbsp;&nbsp;<small><span id="new-label" class=" label label-info">NEW!</span></small>');

        //
        // Create hive payload var from POST fields.
        // Sets hive 'payload' var with payload or empty.
        // 
        if ( $f3->get('VERB') == 'POST' )
        {
            $pl = $f3->get('POST');
            // Remove protected fields from the payload.
            if ( !empty($pl) )
                foreach ($pl as $key => $value)
                    if ( in_array($key, $f3->get('DB_NOCOPY')) )
                        unset($pl[$key]);

            // Put the validated payload in the hive.
            $f3->set('payload', $pl);
        }


        ///////////////////////////////////////////////////////////////////////////////
        // Create connection to database as almost every call requires the database. //
        ///////////////////////////////////////////////////////////////////////////////
        // $f3->set('DB',new DB\SQL($f3->get('DB_CONNECT'), $f3->get('DB_USER'), $f3->get('DB_PWD')));
        // $f3->set('DB_TRIAL', new DB\SQL($f3->get('DB_CONNECT_TRIAL'), $f3->get('DB_USER_TRIAL'), $f3->get('DB_PWD_TRIAL')));

        ////////////////////////////////////////////////////////////
        // Check and authorize for Cyfe dashboard data inquiries. //
        ////////////////////////////////////////////////////////////
        // if ( (stripos($f3->get('URI'), '/datasets/') !== FALSE) )
        // {
        //     $ck = (empty($_GET['custom_key'])) ? '' : $_GET['custom_key'];
        //     if ( empty($ck) )
        //     {
        //         echo 'Unauthorized access.  Action recorded';
        //         exit;
        //     }

        //     // Check for our security token.
        //     $dk = $f3->get('DASHBOARD_KEY');
        //     if ( $ck <> $dk )
        //     {
        //         echo "Unauthorized access.  Action recorded.";
        //         // Sorry you get no data!
        //         exit;
        //     }
        //     // If dates are supplied, put them into the hive.
        //     if ( !empty($_GET['start_date']) )
        //         $f3->set('dashboard_startdate', $_GET['start_date']);
        //     if ( !empty($_GET['end_date']) )
        //         $f3->set('dashboard_startdate', $_GET['end_date']);

        //     // Instantiate the encoder we will need.
        //     $f3->set('ENCODER', new D99EncryptionAgent($f3->get('SECRET')));

        // }


        ////////////////////////
        // Setup Mono Loggers //
        ////////////////////////
       //Create the logger
        // $f3->set('logger', new Logger($f3->get('logger_name')));

        // // Now add some handlers
        // $f3->get('logger')->pushHandler(new RotatingFileHandler($f3->get('MONOLOG'), Logger::DEBUG));
        // $f3->get('logger')->pushHandler(new LogEntriesHandler('77801f61-7419-44a4-98af-f6152aadab8a'));
        //$f3->get('logger')->pushHandler(new PushoverHandler($f3->get('pushover_user_key'), $f3->get('pushover_api_key'), 'OpenFFP Panel'));
    }



    public function getDBMapName($strwithkey='')
    {
        $f3 = \Base::instance();
        $dbmap = '';
        foreach ($f3->get('DBKEYS') as $key => $value)
        {
            if ( stripos($strwithkey, $key) !== FALSE )
            {
                $dbmap = $f3->get('DBKEYS')[$key];
                break;
            }
        }
        return $dbmap;
    }


    /**
     * Tasks required AFTER ALL routes have been followed.
     * Outputs JSON formatted responses from $f3->get('response_status')
     * and $f3->get('response_data').
     * @param  object   $f3 Framework variable
     * @return void
     */
    function afterRoute($f3)
    {
        ////////////////////
        // Log API access //
        ////////////////////
        // Add all usage to api log.
        // $this->addToApiLog();

        // Add all usage to Logentries app log
        // $this->addToAppLog();

        // // See if status exists.
        // $rs = $f3->get('response_status');
        // $rd = $f3->get('response_data');
        // $stat = empty($rs) ? 200 : $f3->get('response_status');

        // // Render the output array into output format.
        // switch ($f3->get('OUTPUT_FORMAT'))
        // {
        //     case 'JSON':
        //         header("Content-Type: application/json", true, $stat);
        //         if ( !empty($rd) )
        //             echo json_encode($f3->get('response_data'));
        //         break;

        //     default:
        //         // See https://github.com/openlss/lib-array2xml for XML format output library.
        //         # code...
        //         break;
        // }
    }

    //     //
    //     // Restrict IP for some actions.
    //     // These actions are usually called with file_get_contents() method on the hosting server.
    //     //

    //     // The property secret key should not be able to be retrieved on any other machine.
    //     $IP_exception['get_prop_instruction_key'] = $_SERVER['SERVER_ADDR'];

    //     // Check the IP_exception array for unauthorized IP addresses.
    //     if ( array_key_exists($action, $IP_exception) and ( strpos($IP_exception[$action], $this->clientIP() ) === FALSE ) )
    //     {
    //         // echo $this->clientIP();
    //         return DooUrlBuilder::url2('ErrorController', 'show_error', array('eid'=>'badip'), true);
    //     }

    // }

}
