<?php

/**
 * AppController class file serving as a base class for all controllers.
 *
 *
 * @author David M. Willink <developer99@ebox99.com>
 * @link http://www.developer99.com/
 * @copyright Copyright &copy; 2015 David M. Willink
 */
use Monolog\Logger;
use Monolog\Handler\RotatingFileHandler;
// use Monolog\Handler\LogEntriesHandler;
// use Monolog\Handler\PushoverHandler;
// Paper Trail Logging
// Sends notifications to Slack
// Sends alerts to Slack and Pushover.net
use Monolog\Formatter\LineFormatter;
use Monolog\Handler\SyslogUdpHandler;

// $sourcePath = '/path/to/source/css/file.css';
// $minifier = new Minify\CSS($sourcePath);
// // we can even add another file, they'll then be
// // joined in 1 output file
// $sourcePath2 = '/path/to/second/source/css/file.css';
// $minifier->add($sourcePath2);
// // or we can just add plain CSS
// $css = 'body { color: #000000; }';
// $minifier->add($css);
// // save minified file to disk
// $minifiedPath = '/path/to/minified/css/file.css';
// $minifier->minify($minifiedPath);
// // or just output the content
// echo $minifier->minify();
use MatthiasMullie\Minify;

// Browser Capability for PHP
// This is a userland replacement for PHP's native get_browser() function, 
// which is officially supported by the Browser Capabilities Project.
// Use the https://github.com/browscap/browscap-php with http://browscap.org/
use phpbrowscap\Browscap;

//
//
//
class AppController
{
    /**
     * Tasks required before ANY route is followed.
     * @param  object   $f3 Framework variable
     * @return void
     */
    function beforeRoute($f3)
    {
        ////////////////////////
        // Setup Mono Loggers //
        ////////////////////////
       //Create the logger
        $f3->set('logger', new Logger($f3->get('logger_name')));

        // // Now add some handlers
        $f3->get('logger')->pushHandler(new RotatingFileHandler($f3->get('MONOLOG'), Logger::DEBUG));
        // $f3->get('logger')->pushHandler(new PushoverHandler($f3->get('pushover_user_key'), $f3->get('pushover_api_key'), $f3->get('logger_name')));
        // $f3->get('logger')->pushHandler(new LogEntriesHandler('77801f61-7419-44a4-98af-f6152aadab8a'));
    
        // Paper Trail Logging - from their help docs.
        // Set the format
        $output = "%channel%.%level_name%: %message%";
        $formatter = new LineFormatter($output);

        // Setup the logger
        // $logger = new Logger('my_logger');
        $syslogHandler = new SyslogUdpHandler("logs2.papertrailapp.com", 14368);
        $syslogHandler->setFormatter($formatter);
        $f3->get('logger')->pushHandler($syslogHandler);

        // Use the new logger
        // $f3->get('logger')->addDebug('DEBUG (100): Detailed debug information.');
        // $f3->get('logger')->addInfo('INFO (200): Interesting events. Examples: User logs in, SQL logs.');
        // $f3->get('logger')->addNotice('NOTICE (250): Normal but significant events.');
        // $f3->get('logger')->addWarning('WARNING (300): Exceptional occurrences that are not errors. Examples: Use of deprecated APIs, poor use of an API, undesirable things that are not necessarily wrong.');
        // $f3->get('logger')->addError('ERROR (400): Runtime errors that do not require immediate action but should typically be logged and monitored.');
        // $f3->get('logger')->addCritical('CRITICAL (500): Critical conditions. Example: Application component unavailable, unexpected exception.');
        // $f3->get('logger')->addAlert('ALERT (550): Action must be taken immediately. Example: Entire website down, database unavailable, etc. This should trigger the SMS alerts and wake you up.');
        // $f3->get('logger')->addEmergency('EMERGENCY (600): Emergency: system is unusable.');

        // $f3->get('logger')->addDebug('DEBUG (100): Detailed debug information. ' . json_encode(array('abc' => 'test', 'def' => 'msg1')));


        // Make sure when accessing this app that we are in secured mode.
        // TODO

        // Encryption Agent needed here.
        $f3->set('encrypt_agent', new EncryptionAgent($f3->get('CRYPT_KEY')));
        $ea = $f3->get('encrypt_agent');

        //
        // Page timer .js is added to every page for now.
        // 
        $r = file_get_contents($f3->get('GLOBAL_ASSETS') . 'vendor/local/js/pagetimer.js');
        $r = str_ireplace('||||SessionExpire||||', $f3->get('PAGE_TIMEOUT') ,$r);
        $r = str_ireplace('||||LogoutURL||||', $f3->get('LEGACY_LOGOUT_URL') ,$r);
        $minify = new Minify\JS($r);
        $f3->set('page_timer_js', $minify->minify());

        // SET COOKIES FOR REQUEST COMING FROM LEGACY APP.
        // AND REDIRECT TO REQUESTED PAGE.
        // $f3->route('GET /legacyredir/@pagename/@appcookie', 'ReportController->viewReportsPage');
        // All /legacyredir/ routes will include user cookie info
        // to digest before launching page.
        if ( !(strpos($f3->get('URI'), 'legacyredir_Wf7Gb3B3xtyne') === FALSE) )
        {
            // See if we are coming from legacy IIS online app.
            // Set user cookies if so.
            if ( !empty($f3->get('PARAMS.appcookie')) )
            {
                $keys = explode('~', $f3->get('PARAMS.appcookie'));
                // ChainID
                $f3->set('COOKIE.userchain', $ea->encode(substr($keys[0], 0, -1)));
                // Chain Code ( ie. hmh )
                $f3->set('COOKIE.chaincode', $ea->encode(substr($keys[1], 0, -1)));
                // Session code string ( userid, sessionid, userpass, userlogin, currenttime, PIN (optional) )
                $f3->set('COOKIE.userid', $ea->encode(substr($keys[2], 0, -1)));
                // User name - Login name
                $f3->set('COOKIE.username', $ea->encode(substr($keys[3], 0, -1)));
                // User full name (first, last)
                $f3->set('COOKIE.userpwd', $ea->encode(substr($keys[4], 0, -1)));
                // User PIN for easy login
                $f3->set('COOKIE.userpin', $ea->encode(substr($keys[5], 0, -1)));
                // User full name (first, last)
                $f3->set('COOKIE.userfullname', $ea->encode(substr($keys[6], 0, -1)));
                // Signal a legacy request
                $f3->set('COOKIE.legacypagerequest', $ea->encode($f3->get('PARAMS.pagename')));
            }

            // Set cookies from any additional data sent in the request.
            $xdata = $f3->get('PARAMS.xferdata');
            if ( !empty($xdata) and (strtoupper($xdata) != 'NONE') )
            {
                // Not sure how this works.
                // http://stackoverflow.com/questions/15966693/explode-into-key-value-pair
                $datapairs = explode('|', $xdata);
                $xferdata = array();
                foreach ($datapairs as $pairindex => $pairdata)
                {
                    list($key, $val) = explode('~', $pairdata);
                    $xferdata = array_merge($xferdata, array($key => $val));
                }
                $f3->set('COOKIE.xferdata', $ea->encode(json_encode($xferdata)));
            }


            // Set cookies from user privilege data sent with request.
            // In the future, this information needs to be a full user context switch.
            $pdata = $f3->get('PARAMS.privilegedata');
            if ( !empty($pdata) and (strtoupper($pdata) != 'NONE') )
            {
                // Not sure how this works.
                // http://stackoverflow.com/questions/15966693/explode-into-key-value-pair
                $datapairs = explode('|', $pdata);
                $privilegedata = array();
                foreach ($datapairs as $pairindex => $pairdata)
                {
                    list($key, $val) = explode('~', $pairdata);
                    $privilegedata = array_merge($privilegedata, array($key => $val));
                }
                $f3->set('COOKIE.privilegedata', $ea->encode(json_encode($privilegedata)) );
            }

            // Get user property information.
            $userpropdata = $f3->get('PARAMS.propdata');
            $userpropdatadecode = urldecode($userpropdata);

            // Encrypt the prop id so that it is not revealed when used publically or in URLs.
            $searchpat = '/("propid":)([0-9]+)(,)/';
            $userpropdata = preg_replace_callback($searchpat, 
                    function ($matches)
                    {
                        $f3 = \Base::instance();
                        $ea = new EncryptionAgent($f3->get('CRYPT_KEY'));
                        return $matches[1] . $matches[2] . $matches[3] . '"propid_enc":"' . $ea->encode($matches[2]) . '",';
                    }, 
                    $userpropdatadecode);

            $upro_decode = json_decode(urldecode($userpropdata), TRUE);
            $f3->set('COOKIE.userprops', $ea->encode(json_encode($upro_decode)));

            // Redirect to the page name requested.
            if ( !empty($f3->get('PARAMS.pagename')) )
            {
                // The pagename is actually the route name.
                // So use the routename to reroute.
                // Since THIS request is to the legacy access point,
                // reset the ALIAS var.  The app uses it to know the page.
                $f3->set('ALIAS', $f3->get('PARAMS.pagename'));
                $f3->reroute('@' . $f3->get('PARAMS.pagename'));
                return;
            }

            // Else bad request.
            $f3->error(400);

        }

        //
        // All non-legacy app requests
        // 
        // Get data passed to this app from a legacy application call.
        $f3->set('xferdata', json_decode($ea->decode($f3->get('COOKIE.xferdata')), TRUE));
        $f3->set('privilegedata', json_decode($ea->decode($f3->get('COOKIE.privilegedata')), TRUE));
        $f3->set('userprops', json_decode($ea->decode($f3->get('COOKIE.userprops')), TRUE));

        // Set global use information to hive.
        $f3->set('userchain', $ea->decode($f3->get('COOKIE.userchain')));
        $f3->set('chaincode', $ea->decode($f3->get('COOKIE.chaincode')));
        $f3->set('userid', $ea->decode($f3->get('COOKIE.userid')));
        $f3->set('username', $ea->decode($f3->get('COOKIE.username')));
        $f3->set('userpwd', $ea->decode($f3->get('COOKIE.userpwd')));
        $f3->set('userpin', $ea->decode($f3->get('COOKIE.userpin')));
        $f3->set('userfullname', $ea->decode($f3->get('COOKIE.userfullname')));

        // So far, all pages are from legacy system and will be for awhile.
        // If no cookies from legacy system, then 404.
        if ( empty($f3->get('COOKIE.chaincode')) )
        {
            $f3->error(404);
            return;
        }

        // Asset base for chosen chain theme
        // Check for chaincode asset theme folder.
        $f3->set('THEME_ASSETS', $f3->get('GLOBAL_ASSETS') . 'theme/' . $f3->get('chaincode') . '/');

        // Make sure main-banner exists or switch to standard theme.
        $r = \Web::instance()->request($f3->get('THEME_ASSETS') . 'images/main-banner.svg' );
        if ( stripos($r['headers'][0], '200 OK') === FALSE )
        {
            $f3->set('THEME_ASSETS', $f3->get('GLOBAL_ASSETS') . 'theme/standard/');
        }

        // Set the global system mode message and set the 
        ///////////////////////////////////////////////////////////////////////////////
        // Create connection to database as almost every call requires the database. //
        ///////////////////////////////////////////////////////////////////////////////
        $f3->set('sys_mode_code', $f3->get('sys_mode_code_prod'));
        $f3->set('sys_mode_desc', $f3->get('sys_mode_desc_prod'));
        if ( ! (stripos($f3->get('URI'), '/demo/') === FALSE) )
        {
            $f3->set('sys_mode_code', $f3->get('sys_mode_code_demo'));
            $f3->set('sys_mode_desc', $f3->get('sys_mode_desc_demo'));
            
            // Make active database point to DEMO database.
            $f3->set('DB', new DB\SQL($f3->get('DB_CONNECT_DEMO'), $f3->get('DB_USER_DEMO'), $f3->get('DB_PWD_DEMO'),
                    array(\PDO::ATTR_ERRMODE=>\PDO::ERRMODE_EXCEPTION) ));
        }
        else
        {
            // This may be prod or demo database based on code to determine development mode in globals.
            $f3->set('DB',new DB\SQL($f3->get('DB_CONNECT'), $f3->get('DB_USER'), $f3->get('DB_PWD'),
                    array(\PDO::ATTR_ERRMODE=>\PDO::ERRMODE_EXCEPTION) ));
        }


        // Set SQL date format for mysql
        $f3->set('SQL_DATE', date("Y-m-d H:i:s"));

        // Get browser information.

        $browscap = new Browscap($f3->get('TEMP') . 'browscap/');
        // $browscap->updateCache();
        $f3->set('browser_info', $browscap->getBrowser());
// ~r($f3->get('browser_info'));

        // Set some browser modes (BM__ indicators for the application
        // BM_OS = WIN or NOTWIN
        $f3->set('BM_OS', ( !(stripos($f3->get('browser_info')->Platform, 'WIN') === FALSE) ? 'WIN' : 'NOTWIN') );


        // //
        // // Turn on NEW labels
        // //
        // $td = new DateTime();

        // // NEW Airline Reconciliation Report
        // $sd = new DateTime("2015-10-18");
        // if ( $sd->diff($td)->days < 120 )
        //     $f3->set('new_report_label','&nbsp;&nbsp;<small><span id="new-label" class=" label label-info">NEW!</span></small>');

        //
        // This allows preprocessing POST payload if necessary.
        // Create hive payload var from POST fields.
        //
        if ( $f3->get('VERB') == 'POST' )
        {
            $pl = $f3->get('POST');
            // Put the validated payload in the hive.
            $f3->set('payload', $pl);
        }

        //
        // Crumbs
        // 
        // $f3->set('menu_structure', array(
        //     'Property Admin' => $f3->alias('adminproperty'

        //     ));




        ////////////////////////////////////////////////////////////
        // Check and authorize for Cyfe dashboard data inquiries. //
        ////////////////////////////////////////////////////////////
        // if ( (stripos($f3->get('URI'), '/datasets/') !== FALSE) )
        // {
        //     $ck = (empty($_GET['custom_key'])) ? '' : $_GET['custom_key'];
        //     if ( empty($ck) )
        //     {
        //         echo 'Unauthorized access.  Action recorded';
        //         exit;
        //     }

        //     // Check for our security token.
        //     $dk = $f3->get('DASHBOARD_KEY');
        //     if ( $ck <> $dk )
        //     {
        //         echo "Unauthorized access.  Action recorded.";
        //         // Sorry you get no data!
        //         exit;
        //     }
        //     // If dates are supplied, put them into the hive.
        //     if ( !empty($_GET['start_date']) )
        //         $f3->set('dashboard_startdate', $_GET['start_date']);
        //     if ( !empty($_GET['end_date']) )
        //         $f3->set('dashboard_startdate', $_GET['end_date']);

        //     // Instantiate the encoder we will need.
        //     $f3->set('ENCODER', new D99EncryptionAgent($f3->get('SECRET')));

        // }


    }



    public function getDBMapName($strwithkey='')
    {
        $f3 = \Base::instance();
        $dbmap = '';
        foreach ($f3->get('DBKEYS') as $key => $value)
        {
            if ( stripos($strwithkey, $key) !== FALSE )
            {
                $dbmap = $f3->get('DBKEYS')[$key];
                break;
            }
        }
        return $dbmap;
    }


    /**
     * Tasks required AFTER ALL routes have been followed.
     * Outputs JSON formatted responses from $f3->get('response_status')
     * and $f3->get('response_data').
     * @param  object   $f3 Framework variable
     * @return void
     */
    function afterRoute($f3)
    {
        ////////////////////
        // Log API access //
        ////////////////////
        // Add all usage to api log.
        // $this->addToApiLog();

        // Add all usage to Logentries app log
        // $this->addToAppLog();

        // // See if status exists.
        // $rs = $f3->get('response_status');
        // $rd = $f3->get('response_data');
        // $stat = empty($rs) ? 200 : $f3->get('response_status');

        // // Render the output array into output format.
        // switch ($f3->get('OUTPUT_FORMAT'))
        // {
        //     case 'JSON':
        //         header("Content-Type: application/json", true, $stat);
        //         if ( !empty($rd) )
        //             echo json_encode($f3->get('response_data'));
        //         break;

        //     default:
        //         // See https://github.com/openlss/lib-array2xml for XML format output library.
        //         # code...
        //         break;
        // }
    }

    //     //
    //     // Restrict IP for some actions.
    //     // These actions are usually called with file_get_contents() method on the hosting server.
    //     //

    //     // The property secret key should not be able to be retrieved on any other machine.
    //     $IP_exception['get_prop_instruction_key'] = $_SERVER['SERVER_ADDR'];

    //     // Check the IP_exception array for unauthorized IP addresses.
    //     if ( array_key_exists($action, $IP_exception) and ( strpos($IP_exception[$action], $this->clientIP() ) === FALSE ) )
    //     {
    //         // echo $this->clientIP();
    //         return DooUrlBuilder::url2('ErrorController', 'show_error', array('eid'=>'badip'), true);
    //     }

    // }

}
